function OpenEditDialog(listurl) {
        var dlgOptions = {
            url: listurl,
            width: 2900,
            height: 450,
            autosize: false
        };
        SP.UI.ModalDialog.showModalDialog(dlgOptions);
    }

var sliderBuilder = function($, tag, options) {
    if (!$) {
        console.error("Slider Builder Error", "$ is undefined");
        return;
    }
    if (!tag) {
        console.error("Slider Builder Error", "tag is undefined");
        return;
    }
    if (!options) {
        console.error("Slider Builder Error", "options is undefined");
        return;
    }

    var defaults = {

        // shared
        presentation: "slider",                 // presentation as 'cards' or as 'slider'
        popupArticle: false,                    // whether to open the link of the article as a modal popup or not
        readMoreHtml: "Read more &raquo;",      // text to display in the "read more" link

        // slider
        tabsLocation: "none",                   // location of the tabs: none, left, right
        sliderHeight: null,                     // value to assign to the height of the slider. If this is not set, the plugin will fallback to the height value set from CSS.

        // cards
        cardsPerRow: 3,                         // 1,2,3,4 cards per row
        cardHeight: null,                       // if set, will override the CSS height values for each card. 
                                                // Less than 300px is not supported
    };
    var opts = $.extend(defaults, options);

    if (opts.presentation == "cards" && [1, 2, 3, 4].indexOf(opts.cardsPerRow) == -1)
        opts.cardsPerRow = 3;

    // Private members
    var $pluginElement = $(tag), 
        stopOnClick, 
        ulTop = 0, 
        visibleTabs = 0, 
        definitiveTabHeight = 0;
    // setInterval(function(){ulTop = $('.slides-nav-tabs').offset().top;}, 2); // Old untested code

    // Public members

    this.activate = function () {
        $pluginElement.empty();

        // add CSS class matching presentation option: 'slider' or 'cards'
        $pluginElement.addClass(opts.presentation);

        if (opts.presentation == "slider") {
            $pluginElement.append(
                $('<div />').addClass('flexslider').append($('<ul />').addClass('slides')),
                $('<ul />').addClass('slides-nav-tabs'));
            this.getResources();
            this.bindSlideClicks();
            createSlider();
        } else {
            this.getResources();
            createCards();
        }
    }

    /**
     * Gets content
     */
    this.getResources = function() {

        var intMaximumItems = 8;
        $.noConflict();

        var query = "<Query>" +
                    "<Where>" +
                    "<And>" +
                        "<And>" +
                        "<Neq>" +
                            "<FieldRef Name='ID'/><Value Type='Number'>0</Value>" +
                        "</Neq>" +

                        "<Geq>" +
                        " <FieldRef Name='" + options.varFieldExpiresOn + "' /> " +
                        " <Value Type='DateTime'>" +
                        " <Today OffsetDays='1' />" +
                        "</Value>" +
                        "</Geq>" +
                        "</And>" +
                                "<Leq>" +
                        " <FieldRef Name='" + options.varFieldPublishOn + "' /> " +
                        " <Value IncludeTimeValue='TRUE'  Type='DateTime'>" +
                        " <Today />" +
                        "</Value>" +
                        "</Leq>" +

                        "</And>" +
                    "</Where>" +
                    "<OrderBy>" +
                        "<FieldRef Name='" + options.varFieldOrderBy + "' Ascending='False'/>" +
                    "</OrderBy>" +
                    "</Query>";
        var url = window.location;
        var readFields = "<ViewFields>" +
                            "<FieldRef Name='ID' />" +
                            "<FieldRef Name='" + options.varFieldImage + "' />" +
                            "<FieldRef Name='" + options.varFieldTitle + "' />" +
                            "<FieldRef Name='" + options.varFieldSummary + "' />" +
                            "<FieldRef Name='LinkFilenameNoMenu' />" +
                            "<FieldRef Name='" + options.varAlternateLinkField + "' />" +
                        "</ViewFields>";

        $().SPServices({
            webURL: options.varListPath,
            operation: "GetListItems",
            listName: options.varListName,
            async: false,
            CAMLQuery: query,
            CAMLViewFields: readFields,
            CAMLRowLimit: intMaximumItems,
            completefunc: function (xData, Status) {
				if (Status == "error") {
				    $pluginElement.append('<p>Slider could not be loaded. Check browser console for errors.</p>');
					var errString = $(xData.responseXML).find('errorstring');
					if (errString != undefined) {
						console.error('Error at $().SPServices( ... ) - ' + errString.html());
					} else {
						console.error('Error at $().SPServices( ... ).');
					}
				}
				
                $(xData.responseXML).find("z\\:row, row").each(function () {

                    if (opts.presentation == "slider") {
                        createSlideMockup($(this));
                    } else {
                        createCardMockup($(this));
                    }
                });

            }
        });
    }

    var getUrl = function (data) {
        var id = data.attr("ows_ID"),
            url = "",
            tempurl = "";

        if (options.varListType == 2) {
            var pagefilename = data.attr("ows_LinkFilenameNoMenu");
            url = options.varListPath + "/" + options.varInternalListName + "/" + pagefilename;
        }
        else if (options.varListType == 1) { //BLOG
            url = options.varListPath + "/Lists/Posts/Post.aspx?ID=" + id;   //FOR BLOG
            //url = "javascript:OpenEditDialog('" + options.varListPath + "/Lists/Posts/Post.aspx?ID=" + id + "')";   //FOR BLOG
        }

        if (options.varAlternateLinkField != "") {
            tempurl = data.attr("ows_" + options.varAlternateLinkField);
            if (undefined != tempurl)
                url = tempurl.split(",", 1);
        }

        if (opts.popupArticle == true) {
            url = "javascript:OpenEditDialog('" + url + "')";
        }

        return url;
    };
    var getImgSrc = function (data) {
        if (options.varListType == 2) {
            return $(data.attr("ows_" + options.varFieldImage)).attr('src');
        }
        else if (options.varListType == 1) { //BLOG
            return $(data.attr("ows_" + options.varFieldImage)).find('img').attr('src');
        }
        return "";
    };
    
    /**
     * Creates Cards and customizes it
     */
    var createCards = function () {
        switch (opts.cardsPerRow) {
            case 1:
                $pluginElement.addClass('card-full');
                break;
            case 2:
                $pluginElement.addClass('card-half');
                break;
            case 3:
                $pluginElement.addClass('card-third');
                break;
            case 4:
            default:
                $pluginElement.addClass('card-quarter');
                break;
        }
    };

    /**
     * Generates the HTML mockup for each card
     */
    var createCardMockup = function (data) {
        var url = getUrl(data),
            imgSrc = getImgSrc(data),
            title = data.attr("ows_" + options.varFieldTitle),
            summary = data.attr("ows_" + options.varFieldSummary);

        /* html parts */
        var $fullCardAnchor = $('<a />').addClass('card-anchor').attr('href', url);

        var $imageWrapper = $('<div />')
                                .addClass('image-wrapper aspect-ratio-16-9')
                                .append($('<img />').attr('src', imgSrc));

        var $cardBody = $('<div />')
                            .addClass('card-body')
                            .append($('<div />').addClass('card-text')
                                .append(
                                    $('<h3 />').text(title),
                                    $('<p />').append(summary)
                                )
                            );

        var $readMore = $('<a />')
                            .addClass('read-more')
                            .attr('href', url).html(opts.readMoreHtml);

        var $cardContent = $('<div />').addClass('card-content');
        // Override the height of each card set from CSS
        if (opts.cardHeight) {
            $cardContent.height(opts.cardHeight);
            if ($cardContent.height() < 300)
                $cardContent.css('height', '');
        }

        /* compiled card item */
        var $cardItem = $('<div />')
                            .addClass('card')
                            .append(
                                $cardContent.append(
                                    $fullCardAnchor,
                                    $imageWrapper,
                                    $cardBody,
                                    $readMore
                                    )
                                );

        /* append card item */
        $cardItem.appendTo(tag);
    };


    /**
     * Creates the Slider and customizes it
     */
    var createSlider = function() {
        $pluginElement
            .find('.flexslider').flexslider({
                slideshow: options.slideshow,
                animation: "fade",
                manualControls: tag + " > .slides-nav-tabs li",
                directionNav: false,
                start: function (slider) { }
            });

        // Override the height of the slider set from CSS
        if (opts.sliderHeight) {
            $(tag + '.slider .slide-image').height(opts.sliderHeight);
        }

        // Layout (bullets/tabs)
        $pluginElement.removeClass("bullets tabs left right");

        switch (opts.tabsLocation) {
            case "left":
            case "right":

                $pluginElement
                    .addClass('tabs')
                    .addClass(opts.tabsLocation);

                /* recalculate item's height once the slider is completed */
                var sliderHeight = $pluginElement.height(),
                    $sliderTabs = $(tag + ' .slides-nav-tabs > li'),
                    //sliderTabsHeight = $(tag + ' .slides-nav-tabs').eq(0).height();
                    sliderTabsHeight = 0;

                $(tag + ' .slides-nav-tabs > li').each(function () {
                    sliderTabsHeight += $(this).height();
                });

                if (sliderTabsHeight < sliderHeight) {
                    // if items do not fill the slider's height, force them to fill it
                    $sliderTabs.height(sliderHeight / $sliderTabs.length);
                }
                else if (sliderTabsHeight > sliderHeight) {
                    // if items are too many, and the last one gets sliced in the middle, adjust height of each so they match slider's full height
                    var itemH = $sliderTabs.eq(0).height();
                    visibleTabs = Math.ceil(sliderHeight / itemH);

                    /* force to fit if an items gets sliced in the middle */
                    if (visibleTabs * itemH > sliderHeight) {
                        $sliderTabs.height(sliderHeight / visibleTabs);
                    };
                }
                definitiveTabHeight = $sliderTabs.eq(0).height();
                visibleTabs = Math.ceil(sliderHeight / definitiveTabHeight);

                /* adjust tabs' width if the slider is too narrow */
                if ($pluginElement.width() > 750) {
                    $pluginElement.addClass('small');
                }

                break;

            case "none":
            default:
                $pluginElement.addClass('bullets');

                if (options.showArrows) {
                    // append next/prev arrows
                    var $customNav = $('<div />').addClass('custom-navigation'),
                        $prev = $('<a />').addClass('flex-prev').attr('href', '#'),
                        $next = $('<a />').addClass('flex-next').attr('href', '#');
                    $prev.append($('<img alt="<" />').attr('src', sliderpath + 'arrow-left.png'));
                    $next.append($('<img alt=">" />').attr('src', sliderpath + 'arrow-right.png'));
                    $customNav.append($prev);
                    $customNav.append($next);
                    $pluginElement.append($customNav);

                    $(tag + ' .flex-prev').on('click', function () {
                        $(tag + " .flexslider").flexslider('prev');
                        return false;
                    });
                    $(tag + ' .flex-next').on('click', function () {
                        $(tag + " .flexslider").flexslider('next');
                        return false;
                    });
                }

                break;
        };
    };

    /**
     * Generates the HTML mockup for each slide
     */
    var createSlideMockup = function (data) {
        var url = getUrl(data), 
            img_src = getImgSrc(data),
            title = data.attr("ows_" + options.varFieldTitle),
            summary = data.attr("ows_" + options.varFieldSummary);

        /* html parts */
        var $imageUrl = $('<a />').attr('href', url);
        var $img_tag = $('<img />').attr('src', img_src);
        var $h1_title = $('<h1 />').append($imageUrl.clone().text(title));
        var $h2_title = $('<h2 />').text(title);
        var $p_summary = $('<p />').append($imageUrl.clone().text(summary));
        var $read_more = $('<a />').attr('href', url).html(opts.readMoreHtml);
        /* compiled slide item */
        var $slide_item = $('<li />').addClass('slide').append(
                $('<div />').addClass('slide-image').attr("style", "background-image:url('" + img_src + "')").append($imageUrl)
            );

        if ("left,right".indexOf(opts.tabsLocation) > -1) {
            $slide_item.append(
                $('<div />').addClass('slide-links').append($read_more)
            );
        } else {
            $slide_item.append(
                $('<div />').addClass('slide-description').append($h1_title, $p_summary)
            );
        };

        /* compiled slide tab */
        var $slide_tab = $('<li />').append(
            $('<div />').addClass('slides-nav-tab-description').append(
                $h2_title,
                $p_summary.clone()
            )
        );

        /* append slide item */
        $slide_item.appendTo(tag + ' .slides');

        /* append slide tab */
        $slide_tab.appendTo(tag + ' .slides-nav-tabs');
    }

    /**
     * Handles slide click events
     */
    this.bindSlideClicks = function () {

        if ("left,right".indexOf(opts.tabsLocation) > -1) {
            
            // vertical slide for slideshow
            setTimeout(function () {
            
                stopOnClick = setInterval(function () {

                    var $activeTab = $pluginElement.find('.slides-nav-tabs > li.flex-active'),
                        $navTabs = $pluginElement.find('.slides-nav-tabs'),
                        currentTabIndex = $activeTab.index(),
                        lastTabIndex = $navTabs.find('li').last().index(),
                        offsetIndex = Math.floor($navTabs.position().top / definitiveTabHeight);

                    if (currentTabIndex == 0) {
                        $navTabs.animate({ 'top': 0 } , 300 );
                    }
                    else if (currentTabIndex + offsetIndex == visibleTabs - 1 && currentTabIndex < lastTabIndex) {
                        $navTabs.animate({ 'top': $navTabs.position().top - definitiveTabHeight }, 300);
                    }
                }, 1000);//interval function ends

            }, 2000);//timeout function ends
        

            // vertical slide on click
            $pluginElement.find('.slides-nav-tabs > li').click(function () {
                clearTimeout(stopOnClick);

                var $currentTab = $(this),
                    $navTabs = $pluginElement.find('.slides-nav-tabs'),
                    currentTabIndex = $currentTab.index(),
                    lastTabIndex = $navTabs.find('li').last().index(),
                    offsetIndex = Math.floor($navTabs.position().top / definitiveTabHeight);
            
                if (currentTabIndex + offsetIndex == 0 && $navTabs.position().top < 0) {
                    $navTabs.animate({ 'top': $navTabs.position().top + definitiveTabHeight }, 300);
                }
                else if (currentTabIndex + offsetIndex == visibleTabs - 1 && currentTabIndex < lastTabIndex) {
                    $navTabs.animate({ 'top': $navTabs.position().top - definitiveTabHeight }, 300);
                }
            });

        }
    }
}

